@@ -112,22 +112,25 @@ const char *RPackage::section()
 
 const char *RPackage::srcPackage()
 {
-   pkgCache::VerIterator ver = _package->VersionList();
+   static string _srcPkg;
+
+   pkgCache::VerIterator ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
    pkgRecords::Parser &rec=_records->Lookup(ver.FileList());
-   const char *s=rec.SourcePkg().empty()?name():rec.SourcePkg().c_str();
+   _srcPkg = rec.SourcePkg().empty()?name():rec.SourcePkg();
  
-   return s;
+   return _srcPkg.c_str();
 }
 
 
 const char *RPackage::summary()
 {
-   pkgCache::VerIterator ver = _package->VersionList();
+   static string _summary;
 
+   pkgCache::VerIterator ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
    if (!ver.end()) {
       pkgRecords::Parser & parser = _records->Lookup(ver.FileList());
-
-      return parser.ShortDesc().c_str();
+      _summary = parser.ShortDesc();
+      return _summary.c_str();
    }
    return "";
 }
@@ -135,11 +138,12 @@ const char *RPackage::summary()
 
 const char *RPackage::maintainer()
 {
-   pkgCache::VerIterator ver = _package->VersionList();
-
+   static string _maintainer;
+   pkgCache::VerIterator ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
    if (!ver.end()) {
       pkgRecords::Parser & parser = _records->Lookup(ver.FileList());
-      return parser.Maintainer().c_str();
+      _maintainer = parser.Maintainer();
+      return _maintainer.c_str();
    }
    return "";
 }
@@ -167,8 +171,7 @@ const char *RPackage::availableVersion()
 
 const char *RPackage::priority()
 {
-   pkgCache::VerIterator ver = _package->VersionList();
-
+   pkgCache::VerIterator ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
    if (ver != 0)
       return ver.PriorityType();
    else
@@ -210,12 +213,13 @@ const char *RPackage::installedFiles()
 
 const char *RPackage::description()
 {
-   pkgCache::VerIterator ver = _package->VersionList();
+   static string _description;
+   pkgCache::VerIterator ver = (*_depcache)[*_package].CandidateVerIter(*_depcache);
 
    if (!ver.end()) {
       pkgRecords::Parser & parser = _records->Lookup(ver.FileList());
-
-      return parseDescription(parser.LongDesc());
+      _description = parseDescription(parser.LongDesc());
+      return _description.c_str();
    } else {
       return "";
    }
@@ -811,7 +815,7 @@ string RPackage::getChangelogFile(pkgAcquire *fetcher)
    if(verstr.find(':')!=verstr.npos)
       verstr=string(verstr, verstr.find(':')+1);
    char uri[512];
-   snprintf(uri,512,"http://changelogs.ubuntu.com/changelogs/pool/%s/%s/%s/%s_%s/changelog",
+   snprintf(uri,512,"http://packages.debian.org/changelogs/pool/%s/%s/%s/%s_%s/changelog",
                                src_section.c_str(),
                                prefix.c_str(),
                                srcpkg.c_str(),
@@ -876,7 +880,7 @@ void RPackage::setPinned(bool flag)
       // delete package from pinning file
       stat(File.c_str(), &stat_buf);
       // create a tmp_pin file in the internal dir
-      string filename = RTmpDir()+"/tmp_pin";
+      string filename = RStateDir() + "/.tmp_preferences";
       FILE *out = fopen(filename.c_str(),"w");
       if (out == NULL)
          cerr << "error opening tmpfile: " << filename << endl;
@@ -1294,7 +1298,7 @@ void RPackageStatus::init()
       markUnsupported = true;
 
       // read supported labels
-      labels = _config->Find("Synaptic::supported-label", "Ubuntu");
+      labels = _config->Find("Synaptic::supported-label", "Debian Debian-Security");
       stringstream sst1(labels);
       while(!sst1.eof()) {
 	 sst1 >> s;
@@ -1302,7 +1306,7 @@ void RPackageStatus::init()
       }
       
       // read supported components
-      components = _config->Find("Synaptic::supported-components", "main updates/main restricted");
+      components = _config->Find("Synaptic::supported-components", "main updates/main");
       stringstream sst2(components);
       while(!sst2.eof()) {
 	 sst2 >> s;
