@@ -36,6 +36,7 @@
 #include <apt-pkg/pkgcache.h>
 #include <apt-pkg/acquire.h>
 #include "rconfiguration.h"
+#include "i18n.h"
 
 using namespace std;
 
@@ -45,6 +46,24 @@ class pkgRecords;
 
 enum { NO_PARSER, DEB_PARSER, STRIP_WS_PARSER, RPM_PARSER };
 
+// taken from apt (pkgcache.cc) to make our life easier 
+// (and added "RDepends" after "Obsoletes"
+static const char *DepTypeStr[] = 
+   {"",_("Depends"),_("PreDepends"),_("Suggests"),
+    _("Recommends"),_("Conflicts"),_("Replaces"),
+    _("Obsoletes"), _("Dependency of")};
+
+typedef struct  {
+   pkgCache::Dep::DepType type; // type as enum
+   const char* name;            // target pkg name
+   const char* version;         // target version  
+   const char* versionComp;     // target version compare type ( << , > etc)
+   bool isSatisfied;            // dependecy is satified 
+   bool isVirtual;              // package is virtual
+   bool isOr;                   // or dependency (with next pkg)
+} DepInformation;
+
+
 class RPackage {
 
    protected:
@@ -55,47 +74,19 @@ class RPackage {
    pkgDepCache *_depcache;
    pkgCache::PkgIterator *_package;
 
+
    // save the default candidate version to undo version selection
    string _defaultCandVer;
 
    bool _notify;
 
    // Virtual pkgs provided by this one.
-#if 0
-   vector<pkgCache::PkgIterator> _virtualPackages;
-   vector<const char *> _provides;
-
-   // Stuff for enumerators
-   int _vpackI;
-   pkgCache::DepIterator _rdepI;
-
-   pkgCache::DepIterator _wdepI;
-   pkgCache::DepIterator _wdepStart;
-   pkgCache::DepIterator _wdepEnd;
-
-   pkgCache::DepIterator _depI;
-   pkgCache::DepIterator _depStart;
-   pkgCache::DepIterator _depEnd;
-
-   bool isWeakDep(pkgCache::DepIterator &dep);
-#endif
    // FIXME: broken right now 
    bool isShallowDependency(RPackage *pkg);
    int _boolFlags;
 
  public:
 
-   typedef struct  {
-      pkgCache::Dep::DepType type; // type as enum
-      const char* typeStr;         // type (depends, preDepends, etc) as str
-      const char* name;            // target pkg name
-      const char* version;         // target version  
-      const char* versionComp;     // target version compare type ( << , > etc)
-      bool isSatisfied;            // dependecy is satified 
-      bool isVirtual;              // package is virtual
-      bool isOr;                   // or dependency (with next pkg)
-   } DepInformation;
-
    enum Flags {
       FKeep             = 1 << 0,
       FInstall          = 1 << 1,
@@ -168,37 +159,6 @@ class RPackage {
    long availableInstalledSize();
    long availablePackageSize();
 
-#if 0
-   // if this is an update
-   UpdateImportance updateImportance();
-   const char *updateSummary();
-   const char *updateDate();
-   const char *updateURL();
-
-   // special case: alway get the deps of the latest available version
-   // (not necessary the installed one)
-   bool enumAvailDeps(const char *&type, const char *&what, const char *&pkg,
-                      const char *&which, char *&summary, bool &satisfied);
-
-   // this gives the dependencies for the installed package
-   vector<RPackage *> getInstalledDeps();
-
-   // installed package if installed, scheduled/candidate if not or if marked
-   bool enumDeps(const char *&type, const char *&what, const char *&pkg,
-                 const char *&which, char *&summary, bool &satisfied);
-   bool nextDeps(const char *&type, const char *&what, const char *&pkg,
-                 const char *&which, char *&summary, bool &satisfied);
-
-   bool enumRDeps(const char *&dep, const char *&what);
-   bool nextRDeps(const char *&dep, const char *&what);
-
-   // weak dependencies
-   bool enumWDeps(const char *&type, const char *&what, bool &satisfied);
-   bool nextWDeps(const char *&type, const char *&what, bool &satisfied);
-
-   void addVirtualPackage(pkgCache::PkgIterator dep);
-#endif
-
    // does the pkg depends on this one?
    bool dependsOn(const char *pkgname);
 
@@ -206,12 +166,14 @@ class RPackage {
    vector<DepInformation> enumDeps(bool useCanidateVersion=false);
 
    // reverse dependencies
-   vector<RPackage::DepInformation> enumRDeps();
+   vector<DepInformation> enumRDeps();
 
    int getFlags();
 
    bool wouldBreak();
 
+   bool isTrusted();
+
    void setKeep();
    void setInstall();
    void setReInstall(bool flag);
