@@ -53,10 +53,15 @@
 #include <apt-pkg/acquire.h>
 #include <apt-pkg/acquire-item.h>
 #include <apt-pkg/clean.h>
+#include <apt-pkg/version.h>
 
 #include <apt-pkg/sourcelist.h>
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/strutl.h>
+#include <apt-pkg/md5.h>
+#ifndef HAVE_RPM
+#include <apt-pkg/debfile.h>
+#endif
 
 #ifdef WITH_LUA
 #include <apt-pkg/luaiface.h>
@@ -273,13 +278,18 @@ bool RPackageLister::upgradable()
    return _cache != NULL && _cache->deps() != NULL;
 }
 
-bool RPackageLister::openCache(bool lock)
+bool RPackageLister::openCache()
 {
    static bool firstRun = true;
 
    // Flush old errors
    _error->Discard();
 
+   // only lock if we run as root
+   bool lock = true;
+   if(getuid() != 0)
+      lock = false;
+
    if (!_cache->open(*_progMeter,lock)) {
       _progMeter->Done();
       _cacheValid = false;
@@ -303,12 +313,11 @@ bool RPackageLister::openCache(bool lock)
    }
 
 #ifdef HAVE_RPM
-   // APT used to have a bug where the destruction order made
-   // this code segfault. APT-RPM has the right fix for this. -- niemeyer
+   // be gentle and free memory
    if (_records)
       delete _records;
 #endif
-
+   
    _records = new pkgRecords(*deps);
    if (_error->PendingError()) {
       _cacheValid = false;
@@ -580,8 +589,8 @@ struct sectionSortFunc {
 // version string compare
 int verstrcmp(const char *x, const char *y)
 {
-   if(x && y)
-      return strcmp(x,y) < 0;
+   if(x && y) 
+      return _system->VS->CmpVersion(x, y) < 0;
    
    // if we compare with a non-existring version
    if(y == NULL)
@@ -607,7 +616,7 @@ struct supportedSortFunc {
    bool _ascent;
    RPackageStatus _status;
  public:
-   supportedSortFunc::supportedSortFunc(bool ascent, RPackageStatus &s) 
+   supportedSortFunc(bool ascent, RPackageStatus &s) 
       : _ascent(ascent), _status(s) {};
    bool operator() (RPackage *x, RPackage *y) {
       if(_ascent)
@@ -778,6 +787,15 @@ void RPackageLister::getDownloadSummary(int &dlCount, double &dlSize)
 {
    dlCount = 0;
    dlSize = _cache->deps()->DebSize();
+
+   pkgAcquire Fetcher;
+   pkgPackageManager *PM = _system->CreatePM(_cache->deps());
+   if (!PM->GetArchives(&Fetcher, _cache->list(), _records)) {
+      delete PM;
+      return;
+   }
+   dlSize = Fetcher.FetchNeeded();
+   delete PM;
 }
 
 
@@ -1199,17 +1217,19 @@ void RPackageLister::getDetailedSummary(vector<RPackage *> &held,
 #ifdef WITH_APT_AUTH
    pkgAcquire Fetcher(NULL);
    pkgPackageManager *PM = _system->CreatePM(_cache->deps());
-   if (!PM->GetArchives(&Fetcher, _cache->list(), _records))
+   if (!PM->GetArchives(&Fetcher, _cache->list(), _records)) {
+      delete PM;
       return;
+   }
    for (pkgAcquire::ItemIterator I = Fetcher.ItemsBegin(); 
 	I < Fetcher.ItemsEnd(); ++I) {
       if (!(*I)->IsTrusted()) {
          notAuthenticated.push_back(string((*I)->ShortDesc()));
       }
    }
-#else
-   sizeChange = deps->UsrSize();
+   delete PM;
 #endif
+   sizeChange = deps->UsrSize();
 }
 
 bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
@@ -1217,11 +1237,7 @@ bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
    assert(_cache->list() != NULL);
    // Get the source list
    //pkgSourceList List;
-#ifdef HAVE_RPM
    _cache->list()->ReadMainList();
-#else
-   _cache->list()->Read(_config->FindFile("Dir::Etc::sourcelist"));
-#endif
    // Lock the list directory
    FileFd Lock;
    if (_config->FindB("Debug::NoLocking", false) == false) {
@@ -1258,10 +1274,15 @@ bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
    if (!_cache->list()->GetIndexes(&Fetcher))
       return false;
 
+// apt-rpm does not support the pulseInterval
+#ifdef HAVE_RPM 
    // Run it
    if (Fetcher.Run() == pkgAcquire::Failed)
       return false;
-
+#else
+   if (Fetcher.Run(50000) == pkgAcquire::Failed)
+      return false;
+#endif
 
 
    //bool AuthFailed = false;
@@ -1282,7 +1303,7 @@ bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
    }
 
    // Clean out any old list files
-   if (_config->FindB("APT::Get::List-Cleanup", true) == true) {
+   if (!Failed && _config->FindB("APT::Get::List-Cleanup", true) == true) {
       if (Fetcher.Clean(_config->FindDir("Dir::State::lists")) == false ||
           Fetcher.Clean(_config->FindDir("Dir::State::lists") + "partial/") ==
           false)
@@ -1297,7 +1318,22 @@ bool RPackageLister::updateCache(pkgAcquireStatus *status, string &error)
    return true;
 }
 
+bool RPackageLister::getDownloadUris(vector<string> &uris)
+{
+   pkgAcquire fetcher;
+   pkgPackageManager *PM = _system->CreatePM(_cache->deps());
+   if (!PM->GetArchives(&fetcher, _cache->list(), _records)) {
+      delete PM;
+      return false;
+   }
+   for (pkgAcquire::ItemIterator I = fetcher.ItemsBegin();
+	I != fetcher.ItemsEnd(); I++) {
+      uris.push_back((*I)->DescURI());
+   }
 
+   delete PM;
+   return true;
+}
 
 bool RPackageLister::commitChanges(pkgAcquireStatus *status,
                                    RInstallProgress *iprog)
@@ -1319,18 +1355,10 @@ bool RPackageLister::commitChanges(pkgAcquireStatus *status,
 
    assert(_cache->list() != NULL);
    // Read the source list
-#ifdef HAVE_RPM
    if (_cache->list()->ReadMainList() == false) {
       _userDialog->
          warning(_("Ignoring invalid record(s) in sources.list file!"));
    }
-#else
-   if (_cache->list()->Read(_config->FindFile("Dir::Etc::sourcelist")) ==
-       false) {
-      _userDialog->
-         warning(_("Ignoring invalid record(s) in sources.list file!"));
-   }
-#endif
 
    pkgPackageManager *PM;
    PM = _system->CreatePM(_cache->deps());
@@ -1344,8 +1372,13 @@ bool RPackageLister::commitChanges(pkgAcquireStatus *status,
    while (1) {
       bool Transient = false;
 
+#ifdef HAVE_RPM
       if (fetcher.Run() == pkgAcquire::Failed)
-         goto gave_wood;
+	 goto gave_wood;
+#else
+      if (fetcher.Run(50000) == pkgAcquire::Failed)
+	 goto gave_wood;
+#endif
 
       string serverError;
 
@@ -1775,4 +1808,70 @@ bool RPackageLister::readSelections(istream &in)
    return true;
 }
 
+bool RPackageLister::addArchiveToCache(string archive, string &pkgname)
+{
+   //cout << "addArchiveToCache() " << archive << endl;
+
+   // do sanity checking on the file (do we need this 
+   // version, arch, or a different one etc)
+   FileFd in(archive, FileFd::ReadOnly);
+   debDebFile deb(in);
+   debDebFile::MemControlExtract Extract("control");
+   if(!Extract.Read(deb)) {
+      cerr << "read failed" << endl;
+      return false;
+   }
+   pkgTagSection tag;
+   if(!tag.Scan(Extract.Control,Extract.Length+2)) {
+      cerr << "scan failed" << endl;
+      return false;
+   }
+   // do we have the pkg
+   pkgname = tag.FindS("Package");
+   RPackage *pkg = this->getPackage(pkgname);
+   if(pkg == NULL) {
+      cerr << "Can't find pkg " << pkgname << endl;
+      return false;
+   }
+   // is it the right architecture?
+   if(tag.FindS("Architecture") != "all" &&
+      tag.FindS("Architecture") != _config->Find("APT::Architecture")) {
+      cerr << "Ignoring different architecture for " << pkgname << endl;
+      return false;
+   }
+   
+   // correct version?
+   string debVer = tag.FindS("Version");
+   string candVer = pkg->availableVersion();
+   if(debVer != candVer) {
+      cerr << "Ignoring " << pkgname << " (different versions: "
+	   << debVer << " != " << candVer  << endl;
+      return false;
+   }
+
+   // md5sum check
+   // first get the md5 of the candidate
+   pkgDepCache *dcache = _cache->deps();
+   pkgCache::VerIterator ver = dcache->GetCandidateVer(*pkg->package());
+   pkgCache::VerFileIterator Vf = ver.FileList(); 
+   pkgRecords::Parser &Parse = _records->Lookup(Vf);
+   string MD5 = Parse.MD5Hash();
+   // then calc the md5 of the pkg
+   MD5Summation debMD5;
+   in.Seek(0);
+   debMD5.AddFD(in.Fd(),in.Size());
+   if(MD5 != debMD5.Result().Value()) {
+      cerr << "Ignoring " << pkgname << " MD5 does not match"<< endl;
+      return false;
+   }
+      
+   // copy to the cache
+   in.Seek(0);
+   FileFd out(_config->FindDir("Dir::Cache::archives")+string(flNotDir(archive)),
+	      FileFd::WriteAny);
+   CopyFile(in, out);
+
+   return true;
+}
+
 // vim:ts=3:sw=3:et
