@@ -125,7 +125,9 @@ void RPackageViewStatus::addPackage(RPackage *pkg)
    _view[str].push_back(pkg);
 
    str.clear();
-   if (flags & RPackage::FNew)
+   if (flags & RPackage::FNowBroken)
+      str = _("Broken dependencies");
+   else if (flags & RPackage::FNew)
       str = _("New in repository");
    else if (flags & RPackage::FPinned) 
       str = _("Pinned");
@@ -166,6 +168,7 @@ void RPackageViewSearch::addPackage(RPackage *pkg)
       break;
    case RPatternPackageFilter::Description:
       str = pkg->name();
+      str += string(pkg->summary());
       str += string(pkg->description());
       break;
    case RPatternPackageFilter::Maintainer:
@@ -173,7 +176,7 @@ void RPackageViewSearch::addPackage(RPackage *pkg)
       break;
    case RPatternPackageFilter::Depends: 
       {
-	 vector<RPackage::DepInformation> d = pkg->enumDeps(true);
+	 vector<DepInformation> d = pkg->enumDeps(true);
 	 for(unsigned int i=0;i<d.size();i++)
 	    str += string(d[i].name);
 	 break; 
@@ -437,7 +440,9 @@ void RPackageViewFilter::makePresetFilters()
       filter->preset = true;
       filter->pattern.addPattern(RPatternPackageFilter::Depends,
                                  "^debconf", false);
-      filter->setName("Pkg with Debconf"); _("Pkg with Debconf");
+      // TRANSLATORS: This is a filter that will give you all packages
+      // with debconf support (that can be reconfigured with debconf)
+      filter->setName("Package with Debconf"); _("Package with Debconf");
       registerFilter(filter);
    }
 #endif
