@@ -72,7 +72,7 @@ bool ShowHelp(CommandLine & CmdL)
       _("-r   Open in the repository screen\n") <<
       _("-f=? Give an alternative filter file\n") <<
       _("-t   Give an alternative main window title (e.g. hostname with `uname -n`)\n") <<
-      _("-i=? Start with the initial Filter with the number given\n") <<
+      _("-i=? Start with the initial Filter with given name\n") <<
       _("-o=? Set an arbitary configuration option, eg -o dir::cache=/tmp\n")<<
       _("--upgrade-mode  Call Upgrade and display changes\n") <<
       _("--dist-upgrade-mode  Call DistUpgrade and display changes\n") <<
@@ -108,6 +108,8 @@ CommandLine::Args Args[] = {
    , {
    0, "ask-cdrom", "Volatile::AskCdrom-Mode", 0}
    , {
+   0, "parent-window-id", "Volatile::ParentWindowId", CommandLine::HasArg}
+   , {
    0, "plug-progress-into", "Volatile::PlugProgressInto", CommandLine::HasArg}
    , {
    0, "progress-str", "Volatile::InstallProgressStr", CommandLine::HasArg} 
@@ -303,6 +305,9 @@ pid_t TestLock(string File)
 //    *) if not, show message and fail
 void check_and_aquire_lock()
 {
+   if (getuid() != 0) 
+      return;
+
    GtkWidget *dia;
    gchar *msg = NULL;
    pid_t LockedApp, runsNonInteractive;
@@ -365,7 +370,7 @@ void check_and_aquire_lock()
 					       GTK_BUTTONS_CLOSE, msg);
       gtk_dialog_run(GTK_DIALOG(dia));
       g_free(msg);
-      exit(0);
+      exit(1);
    }
    
    // we can't get a lock?!?
@@ -397,8 +402,14 @@ int main(int argc, char **argv)
 
    if (getuid() != 0) {
       RGUserDialog userDialog;
-      userDialog.error(_("You must run this program as the root user."));
-      exit(1);
+      userDialog.warning(g_strdup_printf("<b><big>%s</big></b>\n\n%s",
+                                         _("Starting without administrative "
+                                           "privileges"),
+				         _("You will not be able to apply any "
+				           "any changes. But you can still "
+					   "export the marked changes or "
+					   "create a download script "
+					   "for them.")));
    }
 
    if (!RInitConfiguration("synaptic.conf")) {
@@ -541,6 +552,10 @@ int main(int argc, char **argv)
       mainWindow->cbTasksClicked(NULL, mainWindow);
    }
 
+   string filter = _config->Find("Volatile::initialFilter","");
+   if(filter != "")
+      mainWindow->changeView(PACKAGE_VIEW_CUSTOM, filter);
+
    if (NonInteractive) {
       mainWindow->cbProceedClicked(NULL, mainWindow);
    } else {
