@@ -48,82 +48,6 @@
 
 #include "i18n.h"
 
-// timeout in sec until the expander is expanded
-static const int RGTERMINAL_TIMEOUT=60;
-
-// removing
-char* RGDebInstallProgress::remove_stages[NR_REMOVE_STAGES] = {
-   "half-configured", 
-   "half-installed", 
-   "config-files"};
-char* RGDebInstallProgress::remove_stages_translations[NR_REMOVE_STAGES] = {
-   N_("Preparing for removal %s"),
-   N_("Removing %s"),
-   N_("Removed %s")};
-
-// purging
-char *RGDebInstallProgress::purge_stages[NR_PURGE_STAGES] = { 
-   "half-configured",
-   "half-installed", 
-   "config-files", 
-   "not-installed"};
-char *RGDebInstallProgress::purge_stages_translations[NR_PURGE_STAGES] = { 
-   N_("Preparing for removal %s"),
-   N_("Removing with config %s"), 
-   N_("Removed %s"), 
-   N_("Removed with config %s")};
-
-// purge only (for packages that are alreay removed)
-char *RGDebInstallProgress::purge_only_stages[NR_PURGE_ONLY_STAGES] = { 
-   "config-files", 
-   "not-installed"};
-char *RGDebInstallProgress::purge_only_stages_translations[NR_PURGE_ONLY_STAGES] = { 
-   N_("Removing with config %s"), 
-   N_("Removed with config %s")};
-
-// install 
-char *RGDebInstallProgress::install_stages[NR_INSTALL_STAGES] = { 
-   "half-installed",
-   "unpacked",
-   "half-configured",
-   "installed"};
-char *RGDebInstallProgress::install_stages_translations[NR_INSTALL_STAGES] = { 
-   N_("Preparing %s"),
-   N_("Unpacking %s"),
-   N_("Configuring %s"),
-   N_("Installed %s")};
-
-// update
-char *RGDebInstallProgress::update_stages[NR_UPDATE_STAGES] = { 
-   "unpack",
-   "half-installed", 
-   "unpacked",
-   "half-configured",
-   "installed"};
-char *RGDebInstallProgress::update_stages_translations[NR_UPDATE_STAGES] = { 
-   N_("Preparing %s"),
-   N_("Installing %s"), 
-   N_("Unpacking %s"),
-   N_("Configuring %s"),
-   N_("Installed %s")};
-
-//reinstall
-char *RGDebInstallProgress::reinstall_stages[NR_REINSTALL_STAGES] = { 
-   "half-configured",
-   "unpacked",
-   "half-installed", 
-   "unpacked",
-   "half-configured",
-   "installed" };
-char *RGDebInstallProgress::reinstall_stages_translations[NR_REINSTALL_STAGES] = { 
-   N_("Preparing %s"),
-   N_("Unpacking %s"),
-   N_("Installing %s"), 
-   N_("Unpacking %s"),
-   N_("Configuring %s"),
-   N_("Installed %s") };
-
-
 void RGDebInstallProgress::child_exited(VteReaper *vtereaper,
 					gint child_pid, gint ret, 
 					gpointer data)
@@ -138,7 +62,6 @@ void RGDebInstallProgress::child_exited(VteReaper *vtereaper,
    }
 }
 
-
 ssize_t
 write_fd(int fd, void *ptr, size_t nbytes, int sendfd)
 {
@@ -279,7 +202,6 @@ int ipc_recv_fd()
 }
 
 
-
 void RGDebInstallProgress::conffile(gchar *conffile, gchar *status)
 {
    string primary, secondary;
@@ -366,7 +288,6 @@ void RGDebInstallProgress::startUpdate()
 
    // check if we run embedded
    int id = _config->FindI("Volatile::PlugProgressInto", -1);
-   //cout << "Plug ID: " << id << endl;
    if (id > 0) {
       GtkWidget *vbox = glade_xml_get_widget(_gladeXML, "vbox_rgdebinstall_progress");
       _sock =  gtk_plug_new(id);
@@ -390,6 +311,7 @@ void RGDebInstallProgress::cbCancel(GtkWidget *self, void *data)
    
 }
 
+
 void RGDebInstallProgress::cbClose(GtkWidget *self, void *data)
 {
    //cout << "cbCancel: sending SIGKILL to child" << endl;
@@ -423,15 +345,22 @@ bool RGDebInstallProgress::close()
    return TRUE;
 }
 
+RGDebInstallProgress::~RGDebInstallProgress()
+{
+   delete _userDialog;
+}
 
 RGDebInstallProgress::RGDebInstallProgress(RGMainWindow *main,
-					   RPackageLister *lister,
-					   RGUserDialog *userDialog)
+					   RPackageLister *lister)
 
    : RInstallProgress(), RGGladeWindow(main, "rgdebinstall_progress"),
-     _totalActions(0), _progress(0), _sock(0), _userDialog(userDialog)
+     _totalActions(0), _progress(0), _sock(0), _userDialog(0)
 
 {
+   // timeout in sec until the expander is expanded 
+   // (bigger nowdays because of the gconf stuff)
+   _terminalTimeout=_config->FindI("Synaptic::TerminalTimeout",120);
+
    prepare(lister);
    setTitle(_("Applying Changes"));
 
@@ -444,6 +373,10 @@ RGDebInstallProgress::RGDebInstallProgress(RGMainWindow *main,
    _pbarTotal = glade_xml_get_widget(_gladeXML, "progress_total");
    gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(_pbarTotal), 0.025);
    _autoClose = glade_xml_get_widget(_gladeXML, "checkbutton_auto_close");
+   // we don't save options in non-interactive mode, so there is no 
+   // point in showing this here
+   if(_config->FindB("Volatile::Non-Interactive", false))
+      gtk_widget_hide(_autoClose);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_autoClose), 
 				_config->FindB("Synaptic::closeZvt", false));
    //_image = glade_xml_get_widget(_gladeXML, "image");
@@ -480,6 +413,12 @@ RGDebInstallProgress::RGDebInstallProgress(RGMainWindow *main,
    glade_xml_signal_connect_data(_gladeXML, "on_button_close_clicked",
 				 G_CALLBACK(cbClose), this);
 
+   if(_userDialog == NULL)
+      _userDialog = new RGUserDialog(this);
+
+   
+   gtk_window_set_urgency_hint(GTK_WINDOW(_win), FALSE);
+
    // init the timer
    last_term_action = time(NULL);
 }
@@ -513,67 +452,46 @@ void RGDebInstallProgress::updateInterface()
       last_term_action = time(NULL);
 
       if( buf[0] == '\n') {
-// 	 cout << line << endl;
+	 //cout << "got line: " << line << endl;
 	 
-	 gchar **split = g_strsplit(line, ":",4);
-	 
-	 gchar *s=NULL;
+	 gchar **split = g_strsplit(line, ":",5);
+	 gchar *status = g_strstrip(split[0]);
 	 gchar *pkg = g_strstrip(split[1]);
-	 gchar *status = g_strstrip(split[2]);
+	 gchar *percent = g_strstrip(split[2]);
+	 gchar *str = g_strdup(g_strstrip(split[3]));
+
 	 // major problem here, we got unexpected input. should _never_ happen
 	 if(!(pkg && status))
 	    continue;
 
 	 // first check for errors and conf-file prompts
-	 if(strstr(status, "error") != NULL) { 
-	    // error from dpkg
-	    s = g_strdup_printf(_("Error in package %s"), split[1]);
+	 if(strstr(status, "pmerror") != NULL) { 
+	    // error from dpkg, needs to be parsed different
+	    str = g_strdup_printf(_("Error in package %s"), split[1]);
 	    string err = split[1] + string(": ") + split[3];
-	    _error->Error(err.c_str());
-	 } else if(strstr(status, "conffile-prompt") != NULL) {
-	    // conffile-request
+	    _error->Error("%s",utf8(err.c_str()));
+	 } else if(strstr(status, "pmconffile") != NULL) {
+	    // conffile-request from dpkg, needs to be parsed different
 	    //cout << split[2] << " " << split[3] << endl;
 	    conffile(pkg, split[3]);
-	 } else if(_actionsMap.count(pkg) == 0) {
-	    // no known dpkg state (happens e.g if apt reports:
-	    // /bin/sh: apt-listchanges: command-not-found
-	    g_strfreev(split);
-	    line[0] = 0;
-	    continue;
 	 } else {
 	    _startCounting = true;
-
-	    // then go on with the package stuff
-	    char *next_stage_str = NULL;
-	    int next_stage = _stagesMap[pkg];
-	    // is a element is not found in the map, NULL is returned
-	    // (this happens when dpkg does some work left from a previous
-	    //  session (rare but happens))
-	    
-	    char **states = _actionsMap[pkg]; 
-	    char **translations = _translationsMap[pkg]; 
-	    if(states && translations) {
-	       next_stage_str = states[next_stage];
-// 	       cout << "waiting for: " << next_stage_str << endl;
-	       if(next_stage_str && (strstr(status, next_stage_str) != NULL)) {
-		  s = g_strdup_printf(_(translations[next_stage]), split[1]);
-		  next_stage++;
-		  _stagesMap[pkg] = next_stage;
-		  _progress++;
-	       }
-	    }
 	 }
 
-	 // each package goes through various stages
-	 float val = ((float)_progress)/((float)_totalActions);
-// 	 cout << _progress << "/" << _totalActions << " = " << val << endl;
+	 // reset the urgency hint, something changed on the terminal
+	 if(gtk_window_get_urgency_hint(GTK_WINDOW(_win)))
+	    gtk_window_set_urgency_hint(GTK_WINDOW(_win), FALSE);
+
+	 float val = atof(percent)/100.0;
+ 	 //cout << "progress: " << val << endl;
 	 gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(_pbarTotal), val);
-	 if(s!=NULL)
-	    gtk_label_set(GTK_LABEL(_label_status),s);
+
+	 if(str!=NULL)
+	    gtk_label_set(GTK_LABEL(_label_status),utf8(str));
 	 
 	 // clean-up
 	 g_strfreev(split);
-	 g_free(s);
+	 g_free(str);
 	 line[0] = 0;
       } else {
 	 buf[1] = 0;
@@ -591,13 +509,20 @@ void RGDebInstallProgress::updateInterface()
    }
 
 
-   if ((now - last_term_action) > RGTERMINAL_TIMEOUT) {
-      cout << "no statusfd changes/content updates in terminal for " 
-	   << RGTERMINAL_TIMEOUT << "seconds" << endl;
+   if ((now - last_term_action) > _terminalTimeout) {
+      // get some debug info
+      gchar *s;
+      gtk_label_get(GTK_LABEL(_label_status), &s);
+      g_warning("no statusfd changes/content updates in terminal for %i" 
+		" seconds",_terminalTimeout);
+      g_warning("TerminalTimeout in step: %s", s);
+      // now expand the terminal
       GtkWidget *w;
       w = glade_xml_get_widget(_gladeXML, "expander_terminal");
       gtk_expander_set_expanded(GTK_EXPANDER(w), TRUE);
       last_term_action = time(NULL);
+      // try to get the attention of the user
+      gtk_window_set_urgency_hint(GTK_WINDOW(_win), TRUE);
    } 
 
 
@@ -621,9 +546,8 @@ pkgPackageManager::OrderResult RGDebInstallProgress::start(RPackageManager *pm,
    if (res == pkgPackageManager::Failed)
        return res;
 
-   /*
-    * This will make a pipe from where we can read child's output
-    */
+   // we need to send the fds from the pipe over a socket because
+   // ""!%&/ vte_terminal closes all our FDs 
    _child_id = vte_terminal_forkpty(VTE_TERMINAL(_term),NULL,NULL,
 				    false,false,false);
    if (_child_id == 0) {
@@ -631,15 +555,17 @@ pkgPackageManager::OrderResult RGDebInstallProgress::start(RPackageManager *pm,
       pipe(fd);
       ipc_send_fd(fd[0]); // send the read part of the pipe to the parent
 
-#ifdef WITH_DPKG_STATUSFD
       res = pm->DoInstallPostFork(fd[1]);
-#else
-      res = pm->DoInstallPostFork();
-#endif
 
       // dump errors into cerr (pass it to the parent process)	
       _error->DumpErrors();
 
+      // HACK: try to correct the situation
+      if(res == pkgPackageManager::Failed) {
+	 cerr << _("A package failed to install.  Trying to recover:") << endl;
+	 system("dpkg --configure -a");
+      }
+
       ::close(fd[0]);
       ::close(fd[1]);
 
@@ -668,6 +594,7 @@ pkgPackageManager::OrderResult RGDebInstallProgress::start(RPackageManager *pm,
    finishUpdate();
 
    ::close(_childin);
+   _config->Clear("APT::Keep-Fds", _childin);
 
    return res;
 }
@@ -688,7 +615,7 @@ void RGDebInstallProgress::finishUpdate()
    if(res == 0) {
       gtk_widget_grab_focus(_closeB);
       if(autoClose)
-	 _updateFinished = True;
+	 _updateFinished = true;
    }
 
    string s = _config->Find("Volatile::InstallFinishedStr",
@@ -767,45 +694,6 @@ void RGDebInstallProgress::prepare(RPackageLister *lister)
    gtk_label_set_markup(GTK_LABEL(l), msg);
    g_free(msg);
 
-   for (unsigned int row = 0; row < lister->packagesSize(); row++) {
-      RPackage *pkg = lister->getPackage(row);
-      int flags = pkg->getFlags();
-      string name = pkg->name();
-
-      if((flags & RPackage::FPurge)&&
-	 ((flags & RPackage::FInstalled)||(flags&RPackage::FOutdated))){
-	 _actionsMap.insert(pair<string,char**>(name, purge_stages));
-	 _translationsMap.insert(pair<string,char**>(name, purge_stages_translations));
-	 _stagesMap.insert(pair<string,int>(name, 0));
-	 _totalActions += NR_PURGE_STAGES;
-      } else if((flags & RPackage::FPurge)&& 
-		(!(flags & RPackage::FInstalled)||(flags&RPackage::FOutdated))){
-	 _actionsMap.insert(pair<string,char**>(name, purge_only_stages));
-	 _translationsMap.insert(pair<string,char**>(name, purge_only_stages_translations));
-	 _stagesMap.insert(pair<string,int>(name, 0));
-	 _totalActions += NR_PURGE_ONLY_STAGES;
-      } else if(flags & RPackage::FRemove) {
-	 _actionsMap.insert(pair<string,char**>(name, remove_stages));
-	 _translationsMap.insert(pair<string,char**>(name, remove_stages_translations));
-	 _stagesMap.insert(pair<string,int>(name, 0));
-	 _totalActions += NR_REMOVE_STAGES;
-      } else if(flags & RPackage::FNewInstall) {
-	 _actionsMap.insert(pair<string,char**>(name, install_stages));
-	 _translationsMap.insert(pair<string,char**>(name, install_stages_translations));
-	 _stagesMap.insert(pair<string,int>(name, 0));
-	 _totalActions += NR_INSTALL_STAGES;
-      } else if(flags & RPackage::FReInstall) {
-	 _actionsMap.insert(pair<string,char**>(name, reinstall_stages));
-	 _translationsMap.insert(pair<string,char**>(name, reinstall_stages_translations));
-	 _stagesMap.insert(pair<string,int>(name, 0));
-	 _totalActions += NR_REINSTALL_STAGES;
-      } else if((flags & RPackage::FUpgrade)||(flags & RPackage::FDowngrade)) {
-	 _actionsMap.insert(pair<string,char**>(name, update_stages));
-	 _translationsMap.insert(pair<string,char**>(name, update_stages_translations));
-	 _stagesMap.insert(pair<string,int>(name, 0));
-	 _totalActions += NR_UPDATE_STAGES;
-      }
-   }
 }
 
 #endif
