@@ -50,20 +50,76 @@ const char *RGPreferencesWindow::column_visible_names[] =
 const gboolean RGPreferencesWindow::column_visible_defaults[] = 
    { TRUE, TRUE, TRUE, FALSE, FALSE, TRUE, TRUE, TRUE, FALSE, TRUE }; 
 
-void RGPreferencesWindow::onArchiveSelection(GtkWidget *self, void *data)
+#if !GTK_CHECK_VERSION(2,6,0)
+gchar *
+gtk_combo_box_get_active_text (GtkComboBox *combo_box)
 {
-   RGPreferencesWindow *me =
-      (RGPreferencesWindow *) g_object_get_data(G_OBJECT(self), "me");;
+  GtkTreeIter iter;
+  gchar *text = NULL;
+
+  GtkTreeModel* model = gtk_combo_box_get_model(combo_box);
+
+  if (gtk_combo_box_get_active_iter (combo_box, &iter))
+    gtk_tree_model_get (model, &iter, 0, &text, -1);
+
+  return text;
+}
+#endif
+
+void RGPreferencesWindow::cbHttpProxyEntryChanged(GtkWidget *self, void *data)
+{
+   // this function strips http:// from a entred proxy url
+   RGPreferencesWindow *me = (RGPreferencesWindow *) data;
+   const gchar *text = gtk_entry_get_text(GTK_ENTRY(self));
+   gchar *new_text = NULL;
+   if(g_str_has_prefix(text, "http://")) {
+      new_text = g_strdup_printf("%s", &text[strlen("http://")]);
+      gtk_entry_set_text(GTK_ENTRY(self), new_text);
+   }
+   if(new_text != NULL)
+      g_free(new_text);
+}
+
+void RGPreferencesWindow::cbArchiveSelection(GtkWidget *self, void *data)
+{
+   RGPreferencesWindow *me = (RGPreferencesWindow *) data;
    //cout << "void RGPreferencesWindow::onArchiveSelection()" << endl;
    //cout << "data is: " << (char*)data << endl;
 
    if(me->_blockAction)
       return;
 
-   me->_defaultDistro = (char *)data;
-   me->distroChanged = true;
+   gchar *s=gtk_combo_box_get_active_text(GTK_COMBO_BOX(self));
+   if(s!=NULL) {
+      me->_defaultDistro = s;
+      //cout << "new default distro: " << me->_defaultDistro << endl;
+      me->distroChanged = true;
+   }
 }
 
+void RGPreferencesWindow::cbRadioDistributionChanged(GtkWidget *self, 
+						     void *data)
+{
+   //cout << "void RGPreferencesWindow::cbRadioDistributionChanged()" << endl;
+   RGPreferencesWindow *me = (RGPreferencesWindow *) data;
+   
+   // we are only interessted in the active one
+   if(me->_blockAction || !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(self))) 
+      return;
+   
+   gchar *defaultDistro = (gchar*)g_object_get_data(G_OBJECT(self),"defaultDistro");
+   if(strcmp(defaultDistro, "distro") == 0) {
+      gtk_widget_set_sensitive(GTK_WIDGET(me->_comboDefaultDistro),TRUE);
+      gchar *s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(me->_comboDefaultDistro));
+      if(s!=NULL)
+	 me->_defaultDistro = s;
+   } else {
+      gtk_widget_set_sensitive(GTK_WIDGET(me->_comboDefaultDistro),FALSE);
+      me->_defaultDistro = defaultDistro;
+   }
+   //cout << "new default distro: " << me->_defaultDistro << endl;
+   me->distroChanged = true;
+}
 
 void RGPreferencesWindow::applyProxySettings()
 {
@@ -254,7 +310,7 @@ void RGPreferencesWindow::saveColors()
    _config->Set("Synaptic::UseStatusColors", newval ? "true" : "false");
 }
 
-void RGPreferencesWindow::saveTempFiles()
+void RGPreferencesWindow::saveFiles()
 {
    bool newval;
 
@@ -263,7 +319,16 @@ void RGPreferencesWindow::saveTempFiles()
    _config->Set("Synaptic::CleanCache", newval ? "true" : "false");
    newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_cacheAutoClean));
    _config->Set("Synaptic::AutoCleanCache", newval ? "true" : "false");
-   
+
+   // history
+   newval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_delHistory));
+   if(!newval) {
+      _config->Set("Synaptic::delHistory", -1);
+      return;
+   }
+   int delHistory= gtk_spin_button_get_value(GTK_SPIN_BUTTON(_spinDelHistory));
+   _config->Set("Synaptic::delHistory", delHistory);
+   _lister->cleanCommitLog();
 }
 
 void RGPreferencesWindow::saveNetwork()
@@ -299,11 +364,8 @@ void RGPreferencesWindow::saveNetwork()
 
 }
  
-void RGPreferencesWindow::saveExpert()
+void RGPreferencesWindow::saveDistribution()
 {
-   int nr;
-   nr = gtk_option_menu_get_history(GTK_OPTION_MENU(_optionmenuDefaultDistro));
-   
    if (_defaultDistro.empty()) {
       _config->Clear("APT::Default-Release");
       _config->Clear("Synaptic::DefaultDistro");
@@ -321,9 +383,9 @@ void RGPreferencesWindow::saveAction(GtkWidget *self, void *data)
    me->saveGeneral();
    me->saveColumnsAndFonts();
    me->saveColors();
-   me->saveTempFiles();
+   me->saveFiles();
    me->saveNetwork();
-   me->saveExpert();
+   me->saveDistribution();
 
    if (!RWriteConfigFile(*_config)) {
       _error->Error(_("An error occurred while saving configurations."));
@@ -347,9 +409,10 @@ void RGPreferencesWindow::doneAction(GtkWidget *self, void *data)
       me->hide();
       me->_lister->unregisterObserver(me->_mainWin);
       me->_mainWin->setTreeLocked(TRUE);
-      me->_lister->openCache(TRUE);
+      me->_lister->openCache();
       me->_mainWin->setTreeLocked(FALSE);
       me->_lister->registerObserver(me->_mainWin);
+      me->_mainWin->refreshTable();
    }
    me->closeAction(self, data);
 }
@@ -357,13 +420,15 @@ void RGPreferencesWindow::doneAction(GtkWidget *self, void *data)
 void RGPreferencesWindow::changeFontAction(GtkWidget *self, void *data)
 {
    const char *fontName, *propName;
-
+   
    switch (GPOINTER_TO_INT(data)) {
       case FONT_DEFAULT:
          propName = "Synaptic::FontName";
+	      fontName = "sans 10";
          break;
       case FONT_TERMINAL:
          propName = "Synaptic::TerminalFontName";
+	      fontName = "monospace 10";
          break;
       default:
          cerr << "changeFontAction called with unknown argument" << endl;
@@ -430,7 +495,7 @@ void RGPreferencesWindow::readGeneral()
 
    // System upgrade:
    // upgradeType (ask=-1,normal=0,dist-upgrade=1)
-   int i = _config->FindI("Synaptic::upgradeType", -1);
+   int i = _config->FindI("Synaptic::upgradeType", 1);
    gtk_option_menu_set_history(GTK_OPTION_MENU(glade_xml_get_widget(_gladeXML, "optionmenu_upgrade_method")), i + 1);
 
    i = _config->FindI("Synaptic::update::type", 0);
@@ -455,9 +520,11 @@ void RGPreferencesWindow::readGeneral()
    gtk_widget_set_sensitive(GTK_WIDGET(_optionUseTerminal), false);
    _config->Set("Synaptic::UseTerminal", false);
 #else
-#ifndef HAVE_RPM
+#ifndef HAVE_RPM 
+#ifndef WITH_DPKG_STATUSFD
    UseTerminal = true;
 #endif
+#endif
 #endif
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_optionUseTerminal),
                                 _config->FindB("Synaptic::UseTerminal",
@@ -514,18 +581,15 @@ void RGPreferencesWindow::readColors()
 
 }
 
-void RGPreferencesWindow::readTempFiles()
+void RGPreferencesWindow::readFiles()
 {
    // <b>Temporary Files</b>
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cacheClean),
-                                _config->FindB("Synaptic::CleanCache", false));
-
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cacheAutoClean),
-                                _config->FindB("Synaptic::AutoCleanCache",
-                                               false));
-
    bool postClean = _config->FindB("Synaptic::CleanCache", false);
-   bool postAutoClean = _config->FindB("Synaptic::AutoCleanCache", false);
+   bool postAutoClean = _config->FindB("Synaptic::AutoCleanCache", true);
+
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cacheClean), postClean);
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cacheAutoClean), 
+				postAutoClean);
 
    if (postClean)
       gtk_button_clicked(GTK_BUTTON(_cacheClean));
@@ -534,6 +598,15 @@ void RGPreferencesWindow::readTempFiles()
    else
       gtk_button_clicked(GTK_BUTTON(_cacheLeave));
 
+   // history
+   int delHistory = _config->FindI("Synaptic::delHistory", -1);
+   if(delHistory < 0) 
+      gtk_button_clicked(GTK_BUTTON(_keepHistory));
+   else {
+      gtk_button_clicked(GTK_BUTTON(_delHistory));      
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(_spinDelHistory), 
+				delHistory);      
+   }
 }
 
 void RGPreferencesWindow::readNetwork()
@@ -563,45 +636,73 @@ void RGPreferencesWindow::readNetwork()
 
 }
 
-void RGPreferencesWindow::readExpert()
+void RGPreferencesWindow::readDistribution()
 {
    // distro selection, block actions here because the optionmenu changes
    // and a signal is emited
    _blockAction = true;
 
-   distroChanged = false;
-   string defaultDistro = _config->Find("Synaptic::DefaultDistro", "");
    int distroMatch = 0;
-      gtk_option_menu_remove_menu(GTK_OPTION_MENU(_optionmenuDefaultDistro));
+   string defaultDistro = _config->Find("Synaptic::DefaultDistro", "");
    vector<string> archives = _lister->getPolicyArchives();
-   GtkWidget *menu = gtk_menu_new();
-   GtkWidget *mi = gtk_menu_item_new_with_label(_("ignore"));
-   g_object_set_data(G_OBJECT(mi), "me", this);
-   g_signal_connect(G_OBJECT(mi), "activate",
-                    G_CALLBACK(onArchiveSelection), (void *)"");
-   gtk_widget_show(mi);
-   gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-   mi = gtk_separator_menu_item_new();
-   gtk_widget_show(mi);
-   gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+  
+   // setup the toggle buttons
+   GtkWidget *button, *ignore,*now,*distro;
+   ignore = glade_xml_get_widget(_gladeXML, "radiobutton_ignore");
+   g_object_set_data(G_OBJECT(ignore),"defaultDistro",(void*)"");
+   now = glade_xml_get_widget(_gladeXML, "radiobutton_now");
+   g_object_set_data(G_OBJECT(now),"defaultDistro",(void*)"now");
+   distro = glade_xml_get_widget(_gladeXML, "radiobutton_distro");
+   g_object_set_data(G_OBJECT(distro),"defaultDistro",(void*)"distro");
+   glade_xml_signal_connect_data(_gladeXML,
+				 "on_radiobutton_distribution_group_changed",
+				 G_CALLBACK(cbRadioDistributionChanged),
+				 this);
+
+   // clear the combo box
+   GtkTreeModel *model = gtk_combo_box_get_model(GTK_COMBO_BOX(_comboDefaultDistro));
+   int num = gtk_tree_model_iter_n_children(model, NULL);
+   for(;num >= 0;num--)
+      gtk_combo_box_remove_text(GTK_COMBO_BOX(_comboDefaultDistro), num);
+
+   if(defaultDistro == "") {
+      button = ignore;
+      gtk_widget_set_sensitive(GTK_WIDGET(_comboDefaultDistro),FALSE);
+   } else if(defaultDistro == "now") {
+      button = now;
+      gtk_widget_set_sensitive(GTK_WIDGET(_comboDefaultDistro),FALSE);
+   } else {
+      button = distro;
+      gtk_widget_set_sensitive(GTK_WIDGET(_comboDefaultDistro),TRUE);
+   }
+   assert(button);
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+
+
+   // set up combo-box
+   g_signal_connect(G_OBJECT(_comboDefaultDistro), "changed",
+		    G_CALLBACK(cbArchiveSelection), this);
+
+   glade_xml_signal_connect_data(GLADE_XML(_gladeXML),
+				 "on_entry_http_proxy_changed",
+				 G_CALLBACK(cbHttpProxyEntryChanged),
+				 this);
+
    for (unsigned int i = 0; i < archives.size(); i++) {
       //cout << "archive: " << archives[i] << endl;
-      mi = gtk_menu_item_new_with_label(archives[i].c_str());
-      g_object_set_data(G_OBJECT(mi), "me", this);
-      g_signal_connect(G_OBJECT(mi), "activate",
-                       G_CALLBACK(onArchiveSelection),
-                       g_strdup_printf("%s", archives[i].c_str()));
-      gtk_widget_show(mi);
-      gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+      // ignore "now", it's a toggle button item now
+      if(archives[i] == "now")
+	 continue;
+      gtk_combo_box_append_text(GTK_COMBO_BOX(_comboDefaultDistro),
+				 archives[i].c_str());
       if (defaultDistro == archives[i]) {
-         //cout << "match for: " << archives[i] << endl;
-         // i+2 because we have a ignore at pos 0 and a seperator at pos 1
-         distroMatch = i + 2;
+         //cout << "match for: " << archives[i] << " (" << i << ")" << endl;
+	 // we ignored the "now" archive, so we have to subtract by one
+         distroMatch=i-1;
       }
    }
-   gtk_option_menu_set_menu(GTK_OPTION_MENU(_optionmenuDefaultDistro), menu);
-   gtk_option_menu_set_history(GTK_OPTION_MENU(_optionmenuDefaultDistro),
-                               distroMatch);
+   gtk_combo_box_set_active(GTK_COMBO_BOX(_comboDefaultDistro), distroMatch);
+
    _blockAction = false;
 }
 
@@ -610,9 +711,9 @@ void RGPreferencesWindow::show()
    readGeneral();
    readColumnsAndFonts();
    readColors();
-   readTempFiles();
+   readFiles();
    readNetwork();
-   readExpert();
+   readDistribution();
 
    RGWindow::show();
 }
@@ -825,7 +926,7 @@ void RGPreferencesWindow::checkbuttonUserTerminalFontToggled(GtkWidget *self,
 
 RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
                                          RPackageLister *lister)
-: RGGladeWindow(win, "preferences")
+   : RGGladeWindow(win, "preferences"), distroChanged(false)
 {
    GtkWidget *button;
 
@@ -839,8 +940,13 @@ RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
    _optionAskQuit = glade_xml_get_widget(_gladeXML, "check_ask_quit");
    _optionOneClick = glade_xml_get_widget(_gladeXML, "check_oneclick");
 
+   // cache
    _cacheLeave = glade_xml_get_widget(_gladeXML, "radio_cache_leave");
    _cacheClean = glade_xml_get_widget(_gladeXML, "radio_cache_del_after");
+   // history
+   _delHistory = glade_xml_get_widget(_gladeXML, "radio_delete_history");
+   _keepHistory = glade_xml_get_widget(_gladeXML, "radio_keep_history");
+   _spinDelHistory = glade_xml_get_widget(_gladeXML, "spin_del_history");
    _cacheAutoClean =
       glade_xml_get_widget(_gladeXML, "radio_cache_del_obsolete");
    _useProxy = glade_xml_get_widget(_gladeXML, "radio_use_proxy");
@@ -853,9 +959,31 @@ RGPreferencesWindow::RGPreferencesWindow(RGWindow *win,
       glade_xml_get_widget(_gladeXML, "optionmenu_delbutton_action");
    assert(_optionmenuDel);
 
-   _optionmenuDefaultDistro =
-      glade_xml_get_widget(_gladeXML, "optionmenu_default_distro");
-   assert(_optionmenuDefaultDistro);
+   _comboDefaultDistro =
+      glade_xml_get_widget(_gladeXML, "combobox_default_distro");
+   assert(_comboDefaultDistro);
+   GtkTooltips *tips = gtk_tooltips_new();
+   gtk_tooltips_set_tip(GTK_TOOLTIPS(tips),
+			glade_xml_get_widget(_gladeXML,"radiobutton_distro"),
+		       _("Prefer package versions from the selected "
+		       "distribution when upgrading packages. If you "
+		       "manually force a version from a different "
+		       "distribution, the package version will follow "
+		       "that distribution until it enters the default "
+		       "distribution."),"");
+   gtk_tooltips_set_tip(GTK_TOOLTIPS(tips),
+			glade_xml_get_widget(_gladeXML,"radiobutton_now"),
+		       _("Never upgrade to a new version automatically. "
+			    "Be _very_ careful with this option as you will "
+			    "not get security updates automatically! "
+		       "If you manually force a version "
+		       "the package version will follow "
+		       "the choosen distribution."),"");
+   gtk_tooltips_set_tip(GTK_TOOLTIPS(tips),
+			glade_xml_get_widget(_gladeXML,"radiobutton_ignore"),
+			_("Let synaptic pick the best version for you. "
+			"If unsure use this option. "),"");
+
 
    // hide the "remove with configuration" from rpm users
 #ifdef HAVE_RPM
